@@ -18,7 +18,7 @@
 /***
  ***	DNS Query Performance Testing Tool  (queryperf.c)
  ***
- ***	Version $Id: queryperf.c,v 1.1.1.2.2.5.4.4 2006/01/05 02:06:09 marka Exp $
+ ***	Version $Id: queryperf.c,v 1.12 2007/09/05 07:36:04 marka Exp $
  ***
  ***	Stephen Jacob <sj@nominum.com>
  ***/
@@ -39,9 +39,12 @@
 #include <math.h>
 #include <errno.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
 #ifndef HAVE_GETADDRINFO
 #include "missing/addrinfo.h"
 #endif
+#endif
 
 /*
  * Configuration defaults
@@ -217,7 +220,7 @@ void
 show_startup_info(void) {
 	printf("\n"
 "DNS Query Performance Testing Tool\n"
-"Version: $Id: queryperf.c,v 1.1.1.2.2.5.4.4 2006/01/05 02:06:09 marka Exp $\n"
+"Version: $Id: queryperf.c,v 1.12 2007/09/05 07:36:04 marka Exp $\n"
 "\n");
 }
 
